import Public.Standards.vCard;

//! Basic iCalendar objects. Properties can be accessed by the index operator.
class Object {
    //! LowerCaseProperties is inherited from the @[Public.Standards.vCard] module. 
    inherit LowerCaseProperties;

    //! List of Components. Usually contain @[RichComponent] objects.
    array(object) components = ({});

    //! Name of the component or object.
    string name;

    //! Render this object.
    String.Buffer|string render(String.Buffer|void buffer) {
	String.Buffer buf = (buffer) ? buffer : String.Buffer();

	buf->add("BEGIN:"); buf->add(name); buf->add("\r\n");

	::render(buf);

	foreach (components;;object c) {
	    c->render(buf); 
	}

	buf->add("END:"); buf->add(name); buf->add("\r\n");
	
	if (buffer) return buf;
	return buf->get();
    }

    //! Add a component to the object.
    void add_component(object component) {
	components += ({ component });
    }

    string _sprintf(int type) {
	if (type == 'O') {
	    return sprintf("iCalendar.Object(%s, %O %{, %O%})\n", name, properties, components);
	}
    }
}

//! @[Object] subclass that is used for Components. Offers the same Api.
class RichComponent {
    inherit .Object;

    string _sprintf(int type) {
	if (type == 'O') {
	    return sprintf("iCalendar.RichComponent(%s, %O %{, %O%})\n", name, properties, components);
	}
    }
}

class AllSame(object parser) {

    mixed `[](mixed key) {
	return parser;	
    }
}

class AllRich {
    inherit .AllSame;

    void create() {
	::create(.Parser(.RichComponent, .AllPoor()));
    }
}

object nonstrict_parser() {
    return Parser(Object, AllSame(Parser(RichComponent, AllRich())));
}

//! This function should be used to create new parser objects.
object strict_parser() {
    return nonstrict_parser();
}
